const express = require("express");
const http = require("http");
const WebSocket = require("ws");

const app = express();

// BEGIN: HTTP request processing portion of server
app.use(express.static("public")); // To deliver App

app.get("/", function(req, res) {
    res.redirect("/TimeSocketClient.html");
});

// END: HTTP request processing portion of server

// Create HTTP server with Node.js with our express app
// listening for requests
const server = http.createServer(app);

// BEGIN: WebSocket processing portion of server

const wss = new WebSocket.Server({ clientTracking: true, noServer: true });

server.on("upgrade", function(request, socket, head) {
    console.log("Heard a WS upgrade...");
    // Might want to control who can have a socket here
    wss.handleUpgrade(request, socket, head, function(ws) {
        wss.emit("connection", ws, request);
    });
});

let timerMap = new Map();

/*  
    When the connection is finally established
    we receive a WebSocket (ws) to communicate with
    a particular client.
    We can then register to hear messages and such. We
    can also send messages any time we want.
*/
wss.on("connection", function(ws, req) {
    console.log("Server heard something about WS.");
    console.log(
        `Request IP: ${req.connection.remoteAddress}, ${req.connection.remotePort}`
    );
    ws.send("Hi from Time Server!");
    
    let timer = setInterval(function(){
        let dt = new Date();
        ws.send(dt.toString());
        console.log(`There are ${wss.clients.size} clients, sent time ${dt}`);
    }, 5000);
    timerMap.set(ws, timer);
    

    ws.on("message", function(message) {
        console.log(`Received message ${message}`);
        // Send a reply if desired
        ws.send("Got your message");
    });

    ws.on("close", function(code) {
        console.log(`socket closed`);
        clearInterval(timerMap.get(ws));
        timerMap.delete(ws);
    });
});

let port = 2112;
let host = "localhost";
server.listen(port, host, function() {
    console.log(`Time Server started at ${host}, port ${port}.`);
});
